@@ -8,6 +8,7 @@ import abc
 import clu.abstract
 import collections
 import collections.abc
+import contextlib
 import copy
 import os
 
@@ -15,7 +16,7 @@ abstract = abc.abstractmethod
 
 from clu.constants.consts import DEBUG, PROJECT_NAME, NoDefault
 from clu.predicates import attr, tuplize, listify
-from clu.typology import ismapping
+from clu.typology import iterlen, ismapping
 from clu.exporting import Exporter
 
 exporter = Exporter(path=__file__)
@@ -41,6 +42,18 @@ def strip_ns(nskey):
     """ Strip all namespace-related prefixing from a namespaced key """
     return nskey.rpartition(NAMESPACE_SEP)[-1]
 
+def startswith_ns(putative, prefix):
+    """ Boolean predicate to compare a pair of namespace iterables,
+        returning True if the first starts with the second.
+        
+        Do not confuse this with the helper function “compare_ns(…)”,
+        defined below, which returns False if the namespace iterables
+        in question aren’t exactly alike.
+    """
+    putative_ns = concatenate_ns(*putative)
+    prefix_ns = concatenate_ns(*prefix)
+    return putative_ns.startswith(prefix_ns)
+
 @export
 class KeyMapViewBase(collections.abc.Sequence,
                      collections.abc.Sized,
@@ -165,6 +178,91 @@ class KeyMapValuesView(KeyMapViewBase,
                              for nskey in self.mapping \
                               if nskey.startswith(self.prefix))
 
+@export
+class NamespaceWalkerViewBase(KeyMapViewBase):
+    
+    """ A view abstract base class tailored to NamespaceWalker types;
+        specifically, it overrides “__len__(…)” to better utilize
+        the underlying mapping types’ “walk(…)” method.
+    """
+    
+    def __len__(self):
+        if not self.prefix:
+            return iterlen(self.mapping.walk())
+        return iterlen(concatenate_ns(*namespaces) \
+                                  for *namespaces, _, _ in self.mapping.walk() \
+                                   if startswith_ns(namespaces,
+                                               self.namespaces))
+
+@export
+@collections.abc.KeysView.register
+class NamespaceWalkerKeysView(NamespaceWalkerViewBase,
+                              collections.abc.Set):
+    
+    """ A keys view specifically tailored to NamespaceWalker types. """
+    
+    @classmethod
+    def _from_iterable(cls, iterable):
+        # Required by the “collections.abc.Set” API:
+        return set(iterable)
+    
+    def __contains__(self, nskey):
+        for *namespaces, key, value in self.mapping.walk():
+            if startswith_ns(namespaces, self.namespaces):
+                if nskey == pack_ns(key, *namespaces):
+                    return True
+        return False
+    
+    def __iter__(self):
+        for *namespaces, key, value in self.mapping.walk():
+            if startswith_ns(namespaces, self.namespaces):
+                yield pack_ns(key, *namespaces)
+
+@export
+@collections.abc.ItemsView.register
+class NamespaceWalkerItemsView(NamespaceWalkerViewBase,
+                               collections.abc.Set):
+    
+    """ An items view specifically tailored to NamespaceWalker types. """
+    
+    @classmethod
+    def _from_iterable(cls, iterable):
+        # Required by the “collections.abc.Set” API:
+        return set(iterable)
+    
+    def __contains__(self, item):
+        nskey, putative = item
+        for *namespaces, key, value in self.mapping.walk():
+            if putative is value or putative == value:
+                if startswith_ns(namespaces, self.namespaces):
+                    if nskey == pack_ns(key, *namespaces):
+                        return True
+        return False
+    
+    def __iter__(self):
+        for *namespaces, key, value in self.mapping.walk():
+            if startswith_ns(namespaces, self.namespaces):
+                yield (pack_ns(key, *namespaces), value)
+
+@export
+@collections.abc.ValuesView.register
+class NamespaceWalkerValuesView(NamespaceWalkerViewBase,
+                                collections.abc.Collection):
+    
+    """ A values view specifically tailored to NamespaceWalker types. """
+    
+    def __contains__(self, putative):
+        for *namespaces, key, value in self.mapping.walk():
+            if putative is value or putative == value:
+                if startswith_ns(namespaces, self.namespaces):
+                    return True
+        return False
+    
+    def __iter__(self):
+        for *namespaces, key, value in self.mapping.walk():
+            if startswith_ns(namespaces, self.namespaces):
+                yield value
+
 # NAMESPACE-MANIPULATION FUNCTION API:
 
 def validate_ns(*namespaces):
@@ -436,7 +534,7 @@ class FrozenFlat(FrozenKeyMap, clu.abstract.ReprWrapper,
     
     __slots__ = tuplize('dictionary')
     
-    def __init__(self, dictionary=None, *args, **updates):
+    def __init__(self, dictionary=None, **updates):
         """ Initialize a flat KeyMap instance from a target dictionary.
             
             The dictionary can contain normal key-value items as long as the
@@ -444,7 +542,7 @@ class FrozenFlat(FrozenKeyMap, clu.abstract.ReprWrapper,
             output of ‘pack_ns(…)’ (q.v. function definition supra.)
         """
         try:
-            super(FrozenFlat, self).__init__(*args, **updates)
+            super(FrozenFlat, self).__init__(**updates)
         except TypeError:
             super(FrozenFlat, self).__init__()
         if hasattr(dictionary, 'dictionary'):
@@ -504,8 +602,51 @@ class Flat(FrozenFlat, KeyMap):
 @export
 class NamespaceWalker(FrozenKeyMap):
     
+    """ A NamespaceWalker type implements a “walk(…)” method for iterating
+        over namespaced key-value items.
+        
+        In return for furnishing this one method, NamespaceWalkers receive
+        implementations for “__iter__()”, “__len__()”, “__contains__(…)”,
+        and “__getitem__(…)”, plus optimized view-types returned from their
+        “keys(…)”, “items(…)” and “values(…)” calls, a “flatten(…)” method,
+        and an optimized version of the “namespaces()” method, ALL FREE!!
+        
+        For KeyMap types whose backend mechanics are well-suited to being
+        “walked” (as it were) this is a remarkably good deal, would you not
+        agree??
+        
+        See the docstring for “NamespaceWalker.walk(…)” for details. The
+        original “walk(…)” output format and model implementation were 
+        derived from this StackOverflow answer:
+        
+            • https://stackoverflow.com/a/12507546/298171
+    """
+    
     @abstract
     def walk(self):
+        """ The “walk(…)” method backs all other “NamespaceWalker” methods.
+            
+            Concrete subclasses must implement “walk(…)” such that it iterates
+            over all items in a given instance, yielding them in a form like:
+            
+                for *namespaces, key, value in self.walk():
+                    # …
+            
+            … So an item with no namespaces would yield “['key', 'value']”,
+            but one with three would yield “['do', 're', 'me', 'key', 'value']”.
+            
+            See the “mapwalk(…)” and “envwalk(…)” function implementations,
+            for practical examples of how this can work. “mapwalk(…)” iterates
+            over nested dictionaries-of-dictionaries, and “envwalk(…)” transforms
+            the values in an environment dictionary (like ‘os.environ’) into the
+            above namespaced-key-value format, as noted.
+            
+            N.B. Implementors may wish to write their own less-näive versions of
+            “__contains__(…)” and “__getitem__(…)” in their subclasses, depending
+            on how such subclasses work internally – in many cases, implementing
+            these methods using domain-specific logic will be faster and/or less
+            pathological than doing so using only “walk(…)”.
+        """
         ...
     
     def flatten(self, cls=None):
@@ -525,12 +666,39 @@ class NamespaceWalker(FrozenKeyMap):
                 nss.add(concatenate_ns(*namespaces))
         yield from sorted(nss)
     
+    def keys(self, *namespaces, unprefixed=False):
+        """ Return a namespaced view over either all keys in the mapping,
+            or over only those keys in the mapping matching the specified
+            namespace values.
+        """
+        if unprefixed:
+            return self.submap(unprefixed=unprefixed).keys()
+        return NamespaceWalkerKeysView(self, *namespaces)
+    
+    def items(self, *namespaces, unprefixed=False):
+        """ Return a namespaced view over either all key/value pairs in the
+            mapping, or over only those key/value pairs in the mapping whose
+            keys match the specified namespace values.
+        """
+        if unprefixed:
+            return self.submap(unprefixed=unprefixed).items()
+        return NamespaceWalkerItemsView(self, *namespaces)
+    
+    def values(self, *namespaces, unprefixed=False):
+        """ Return a namespaced view over either all values in the mapping,
+            or over only those values in the mapping whose keys match the
+            specified namespace values.
+        """
+        if unprefixed:
+            return self.submap(unprefixed=unprefixed).values()
+        return NamespaceWalkerValuesView(self, *namespaces)
+    
     def __iter__(self):
         for *namespaces, key, value in self.walk():
             yield pack_ns(key, *namespaces)
     
     def __len__(self):
-        return len(tuple(self.walk()))
+        return iterlen(self.walk())
     
     def __contains__(self, nskey):
         key, namespaces = unpack_ns(nskey)
@@ -548,10 +716,12 @@ class NamespaceWalker(FrozenKeyMap):
                     return value
         raise KeyError(nskey)
 
-def DefaultTree():
-    return collections.defaultdict(DefaultTree)
+def DefaultTree(*args, **kwargs):
+    """ Initialize a recursive DefaultDict pseudo-tree. """
+    return collections.defaultdict(DefaultTree, *args, **kwargs)
 
 def dictify(tree):
+    """ Recursively convert a nested mapping to standard dicts. """
     if ismapping(tree):
         return { key : dictify(tree[key]) for key in tree }
     return tree
@@ -583,19 +753,19 @@ class FrozenNested(NamespaceWalker, clu.abstract.ReprWrapper,
     
     __slots__ = tuplize('tree')
     
-    def __init__(self, tree=None, *args, **updates):
+    def __init__(self, tree=None, **updates):
         """ Initialize an articulated (née “nested”) KeyMap instance from a
             target nested dictionary (or a “tree” of dicts).
         """
         try:
-            super(FrozenNested, self).__init__(*args, **updates)
+            super(FrozenNested, self).__init__(**updates)
         except TypeError:
             super(FrozenNested, self).__init__()
         if hasattr(tree, 'tree'):
             tree = attr(tree, 'tree')
         elif hasattr(tree, 'nestify'):
             tree = attr(tree.nestify(), 'tree')
-        self.tree = tree or DefaultTree()
+        self.tree = DefaultTree(tree or {})
         if updates:
             self.tree.update(**updates)
     
@@ -603,6 +773,17 @@ class FrozenNested(NamespaceWalker, clu.abstract.ReprWrapper,
         """ Iteratively walk the nested KeyMap’s tree of dicts. """
         yield from mapwalk(self.tree)
     
+    def submap(self, *namespaces, unprefixed=False):
+        """ Return a standard dict containing only the namespaced items. """
+        # The “unprefixed” codepath is optimized here, the rest
+        # delegates up to the ancestor implementation:
+        if unprefixed:
+            return { key : value for key, value in self.tree.items() \
+                                  if not ismapping(value) }
+        if not namespaces:
+            return dict(self)
+        return super().submap(*namespaces)
+    
     def inner_repr(self):
         return repr(self.tree)
     
@@ -634,14 +815,17 @@ class Nested(FrozenNested, KeyMap):
             d[key] = value
     
     def __delitem__(self, nskey):
-        key, namespaces = unpack_ns(nskey)
-        if not namespaces:
-            del self.tree[key]
+        if nskey in self:
+            key, namespaces = unpack_ns(nskey)
+            if not namespaces:
+                del self.tree[key]
+            else:
+                d = self.tree
+                for namespace in namespaces:
+                    d = d[namespace]
+                del d[key]
         else:
-            d = self.tree
-            for namespace in namespaces:
-                d = d[namespace]
-            del d[key]
+            raise KeyError(nskey)
 
 # ENVIRONMENT-VARIABLE MANIPULATION API:
 
@@ -710,6 +894,7 @@ def nskey_to_env(appname, nskey):
     key, namespaces = unpack_ns(nskey)
     return pack_env(appname, key, *namespaces)
 
+@export
 def envwalk(appname, mapping):
     """ Iteratively walk an environment-variable mapping, selecting
         only the variables prefixed for the given appname, and convert
@@ -734,28 +919,55 @@ class FrozenEnviron(NamespaceWalker, clu.abstract.ReprWrapper,
     
     __slots__ = ('environment', 'appname')
     
-    def __init__(self, environment=None, appname=None, *args, **updates):
+    def __init__(self, environment=None, appname=None, **updates):
         """ Initialize a FrozenKeyMap instance wrapping an environment-variable
             dictionary from a target dictionary, with a supplied appname.
         """
         try:
-            super(FrozenEnviron, self).__init__(*args, **updates)
+            super(FrozenEnviron, self).__init__(**updates)
         except TypeError:
             super(FrozenEnviron, self).__init__()
-        self.environment = environment or os.environ.copy()
         self.appname = appname or PROJECT_NAME
+        self.environment = environment is None \
+                       and os.environ.copy() \
+                        or environment
         if updates:
             self.environment.update(**updates)
     
     def walk(self):
-        """ Iteratively walk the environment access dict. """
+        """ Iteratively walk the backend environment access dictionary. """
         yield from envwalk(self.appname,
                            self.environment)
     
+    def __contains__(self, nskey):
+        envkey = nskey_to_env(self.appname, nskey)
+        return envkey in self.environment
+    
+    def __getitem__(self, nskey):
+        envkey = nskey_to_env(self.appname, nskey)
+        return self.environment[envkey]
+    
+    def hasenv(self, envkey):
+        """ Query the backend environment dictionary for a key. """
+        return envkey in self.environment
+    
+    def getenv(self, envkey, default=NoDefault):
+        """ Retrieve a key directly from the backend environment. """
+        if default is NoDefault:
+            return self.environment[envkey]
+        try:
+            return self.environment[envkey]
+        except KeyError:
+            return default
+    
+    def envkeys(self):
+        """ Get a view on the dictionary keys from the backend environment. """
+        return self.environment.keys()
+    
     def inner_repr(self):
         """ Return some readable meta-information about this instance """
         prefix = prefix_env(self.appname)
-        nscount = len(tuple(self.namespaces()))
+        nscount = iterlen(self.namespaces())
         keycount = len(self.keys())
         return f"[prefix=“{prefix}*”, namespaces={nscount}, keys={keycount}]"
     
@@ -764,9 +976,11 @@ class FrozenEnviron(NamespaceWalker, clu.abstract.ReprWrapper,
         return type(self)(tree=copier(self.tree))
 
 @export
-class Environ(FrozenEnviron, KeyMap):
+class Environ(FrozenEnviron, KeyMap, contextlib.AbstractContextManager):
     
-    def __init__(self, environment=None, appname=None, *args, **updates):
+    __slots__ = tuplize('stash')
+    
+    def __init__(self, environment=None, appname=None, **updates):
         """ Initialize a KeyMap instance wrapping an environment-variable
             dictionary from a target dictionary, with a supplied appname.
         """
@@ -775,10 +989,11 @@ class Environ(FrozenEnviron, KeyMap):
         try:
             super(Environ, self).__init__(environment=environment,
                                               appname=appname,
-                                             *args, **updates)
+                                                    **updates)
         except TypeError:
             super(Environ, self).__init__(environment=environment,
                                               appname=appname)
+        self.stash = None
     
     def freeze(self):
         return FrozenEnviron(environment=self.environment.copy(),
@@ -791,6 +1006,29 @@ class Environ(FrozenEnviron, KeyMap):
     def __delitem__(self, nskey):
         envkey = nskey_to_env(self.appname, nskey)
         del self.environment[envkey]
+    
+    def setenv(self, envkey, value):
+        """ Set the value for a key directly in the backend environment. """
+        self.environment[envkey] = value
+    
+    def unsetenv(self, envkey):
+        """ Delete a key directly from the backend environment """
+        del self.environment[envkey]
+    
+    def __enter__(self):
+        self.stash = self.environment.copy()
+        return self
+    
+    def __exit__(self, exc_type=None,
+                       exc_val=None,
+                       exc_tb=None):
+        if self.stash:
+            try:
+                self.environment.clear()
+            finally:
+                self.environment.update(self.stash)
+        self.stash = None
+        return exc_type is None
 
 export(ENVIRONS_SEP,  name='ENVIRONS_SEP')
 export(NAMESPACE_SEP, name='NAMESPACE_SEP')
@@ -798,12 +1036,9 @@ export(NAMESPACE_SEP, name='NAMESPACE_SEP')
 # Assign the modules’ `__all__` and `__dir__` using the exporter:
 __all__, __dir__ = exporter.all_and_dir()
 
-def test():
-    
-    from clu.testing.utils import inline
-    from pprint import pprint
-    
-    nestedmaps = {'body': {'declare_i': {'id': {'name': 'i', 'type': 'Identifier'},
+def nestedmaps():
+    """ Private nested-dictionary pseudo-fixture """
+    return {'body':   {'declare_i': {'id': {'name': 'i', 'type': 'Identifier'},
                                             'init': {'type': 'Literal', 'value': 2},
                                             'type': 'VariableDeclarator'},
                            'kind': 'var',
@@ -823,16 +1058,32 @@ def test():
                                             'type': 'VariableDeclarator'},
                            'kind': 'var',
                            'type': 'VariableDeclaration'},
-                 'type': 'Program'}
+            'type':     'Program'}
+
+def flatdict():
+    """ Private flat-dictionary pseudo-fixture """
+    out = {}
+    for mappingpath in mapwalk(nestedmaps()):
+        *namespaces, key, value = mappingpath
+        nskey = pack_ns(key, *namespaces)
+        out[nskey] = value
+    return out
+
+def test():
+    
+    from clu.testing.utils import inline
+    from pprint import pprint
     
     @inline
     def test_one():
-        dictderive = tuple(mapwalk(nestedmaps))
+        """ Simple “mapwalk(…)” content """
+        dictderive = tuple(mapwalk(nestedmaps()))
         return dictderive
     
     @inline
     def test_two():
-        for mappingpath in mapwalk(nestedmaps):
+        """ Verbose “mapwalk(…)” content check """
+        for mappingpath in mapwalk(nestedmaps()):
             *namespaces, key, value = mappingpath
             nskey = pack_ns(key, *namespaces)
             print("NAMESPACES:", ", ".join(namespaces))
@@ -843,76 +1094,30 @@ def test():
     
     @inline
     def test_three():
+        """ FrozenFlat and Nested equivalence """
         flat_dict = {}
         
-        for mappingpath in mapwalk(nestedmaps):
+        for mappingpath in mapwalk(nestedmaps()):
             *namespaces, key, value = mappingpath
             nskey = pack_ns(key, *namespaces)
             flat_dict[nskey] = value
         
-        print(f"FLAT DICTIONARY (length={len(flat_dict)}):")
-        pprint(flat_dict, indent=4)
-        print()
-        
         flat = FrozenFlat(flat_dict)
-        
-        print(f"FLAT FROZEN INSTANCE (length={len(flat)}):")
-        pprint(flat)
-        print()
-        
-        keys = tuple(flat.keys())
-        print(f"FLAT KEYS (length={len(keys)}):")
-        pprint(keys)
-        print()
-        
-        values = tuple(flat.values())
-        print(f"FLAT VALUES (length={len(values)}):")
-        pprint(values)
-        print()
-        
-        namespaces = tuple(flat.namespaces())
-        print(f"FLAT NAMESPACES (length={len(namespaces)}):")
-        pprint(namespaces)
-        print()
-        
         nested = Nested(flat)
         assert nested.flatten() == flat
         assert nested == flat
     
     @inline
     def test_three_pt_five():
+        """ Flat, FrozenFlat and Nested equivalence """
         flat_dict = {}
         
-        for mappingpath in mapwalk(nestedmaps):
+        for mappingpath in mapwalk(nestedmaps()):
             *namespaces, key, value = mappingpath
             nskey = pack_ns(key, *namespaces)
             flat_dict[nskey] = value
         
-        print(f"FLAT DICTIONARY (length={len(flat_dict)}):")
-        pprint(flat_dict, indent=4)
-        print()
-        
         flat = Flat(flat_dict)
-        
-        print(f"FLAT MUTABLE INSTANCE (length={len(flat)}):")
-        pprint(flat)
-        print()
-        
-        keys = tuple(flat.keys())
-        print(f"FLAT KEYS (length={len(keys)}):")
-        pprint(keys)
-        print()
-        
-        values = tuple(flat.values())
-        print(f"FLAT VALUES (length={len(values)}):")
-        pprint(values)
-        print()
-        
-        namespaces = tuple(flat.namespaces())
-        print(f"FLAT NAMESPACES (length={len(namespaces)}):")
-        pprint(namespaces)
-        print()
-        
         frozen_flat = flat.freeze()
         assert frozen_flat == flat
         
@@ -922,80 +1127,36 @@ def test():
     
     @inline
     def test_four():
-        nested = FrozenNested(tree=nestedmaps)
+        """ FrozenNested contains namespaced key """
+        nested = FrozenNested(tree=nestedmaps())
         
         for mappingpath in mapwalk(nested.tree):
             *namespaces, key, value = mappingpath
             nskey = pack_ns(key, *namespaces)
-            print("NAMESPACES:", ", ".join(namespaces))
-            print("KEY:", key)
-            print("NSKEY:", nskey)
-            print("VALUE:", value)
-            print()
+            assert nskey in nested
     
     @inline
     def test_four_pt_five():
-        nested = Nested(tree=nestedmaps)
+        """ Nested (mutable) contains namespaced key """
+        nested = Nested(tree=nestedmaps())
         
         for mappingpath in mapwalk(nested.tree):
             *namespaces, key, value = mappingpath
             nskey = pack_ns(key, *namespaces)
-            print("NAMESPACES:", ", ".join(namespaces))
-            print("KEY:", key)
-            print("NSKEY:", nskey)
-            print("VALUE:", value)
-            print()
+            assert nskey in nested
     
     @inline
-    def test_four_pt_seven():
-        nested = FrozenNested(tree=nestedmaps)
-        
-        print(f"NESTED FROZEN INSTANCE (length={len(nested)}):")
-        pprint(nested)
-        print()
-        
-        keys = tuple(nested.keys())
-        print(f"NESTED KEYS (length={len(keys)}):")
-        pprint(keys)
-        print()
-        
-        values = tuple(nested.values())
-        print(f"NESTED VALUES (length={len(values)}):")
-        pprint(values)
-        print()
-        
-        namespaces = tuple(nested.namespaces())
-        print(f"NESTED NAMESPACES (length={len(namespaces)}):")
-        pprint(namespaces)
-        print()
-        
+    def test_five():
+        """ FrozenNested and Flat roundtrip commutativity """
+        nested = FrozenNested(tree=nestedmaps())
         flat = Flat(nested)
         assert flat.nestify() == nested
         assert flat == nested
     
     @inline
-    def test_five():
-        nested = Nested(tree=nestedmaps)
-        
-        print(f"NESTED MUTABLE INSTANCE (length={len(nested)}):")
-        pprint(nested)
-        print()
-        
-        keys = tuple(nested.keys())
-        print(f"NESTED KEYS (length={len(keys)}):")
-        pprint(keys)
-        print()
-        
-        values = tuple(nested.values())
-        print(f"NESTED VALUES (length={len(values)}):")
-        pprint(values)
-        print()
-        
-        namespaces = tuple(nested.namespaces())
-        print(f"NESTED NAMESPACES (length={len(namespaces)}):")
-        pprint(namespaces)
-        print()
-        
+    def test_five_pt_five():
+        """ Nested (mutable) and FrozenNested roundtrip commutativity """
+        nested = Nested(tree=nestedmaps())
         frozen_nested = nested.freeze()
         assert frozen_nested == nested
         
@@ -1005,20 +1166,71 @@ def test():
     
     @inline
     def test_six():
+        """ FrozenEnviron and “envwalk(…)” namespaced-key check """
+        env = FrozenEnviron()
+        
         for *namespaces, key, value in envwalk('clu', os.environ.copy()):
-            print("NAMESPACES:", namespaces)
-            print("KEY:", f"«{key}»")
-            print("VALUE:", f"“{value}”")
-            print()
+            nskey = pack_ns(key, *namespaces)
+            assert nskey in env
+    
+    @inline
+    def test_seven():
+        """ Environ with “os.environ” and custom-dict backends """
+        env = Environ()
+        nenv = env.flatten().nestify()
+        wat = Environ(environment={ nskey_to_env('clu', nskey) : value \
+                                    for nskey, value \
+                                     in nenv.flatten().items() })
+        assert env == wat
+        assert env.flatten() == nenv
+        assert len(env.envkeys()) >= len(env)
+        assert len(wat.envkeys()) == len(wat)
+    
+    @inline
+    def test_eight():
+        """ FrozenEnviron low-level API """
+        env = FrozenEnviron()
+        
+        for key in env.envkeys():
+            assert env.hasenv(key)
+            assert env.getenv(key) == os.getenv(key)
+        
+        # N.B. It looks like the “os.{get,put,unset}env(…)” functions
+        # don’t really fucking work the way they should:
+        try:
+            before = len(env)
+            os.environ['CLU_CTX_YODOGG'] = 'I heard you are frozen'
+            assert len(env) == before
+            assert not env.hasenv('CLU_CTX_YODOGG')
+            assert os.getenv('CLU_CTX_YODOGG') == 'I heard you are frozen'
+            # print("CLU_CTX_YODOGG:", os.getenv('CLU_CTX_YODOGG'))
+        finally:
+            # os.unsetenv('CLU_CTX_YODOGG')
+            del os.environ['CLU_CTX_YODOGG']
+    
+    @inline
+    def test_eight_pt_five():
+        """ Environ (mutable) context-manager API """
+        before = len(os.environ)
+        assert os.getenv('CLU_CTX_YODOGG') is None
+        
+        with Environ() as env:
+            env.set('yodogg', 'I heard you like managed context', 'ctx')
+            assert env.getenv('CLU_CTX_YODOGG') == 'I heard you like managed context'
+            assert os.getenv('CLU_CTX_YODOGG') == 'I heard you like managed context'
+            assert len(os.environ) == before + 1
+        
+        # Why can we still access the thing in unmanaged scope???
+        assert env.stash is None
+        
+        assert os.getenv('CLU_CTX_YODOGG') is None
+        assert len(os.environ) == before
     
     print()
     pprint(nestedmaps)
     
     # Run all inline tests:
-    inline.test(vars(), 100)
-    # inline.test(vars())
-    # test_five()
-    # test_six()
+    inline.test(100)
 
 if __name__ == '__main__':
     test()
