@@ -9,7 +9,7 @@ import contextlib
 import os
 import weakref
 
-from clu.constants.consts import DEBUG, NoDefault
+from clu.constants.consts import NoDefault
 from clu.constants.polyfills import Path
 from clu.abstract import Slotted
 from clu.config.abc import NAMESPACE_SEP, functional_and, functional_set
@@ -909,8 +909,6 @@ class NamespacedFieldManager(object):
 def __getattr__(key):
     """ Module __getattr__(…) instances the NamespacedFieldManager on-demand """
     if key == 'fields':
-        if DEBUG:
-            print("» Instancing NamespacedFieldManager…")
         return NamespacedFieldManager()
     raise AttributeError(f"module {__name__} has no attribute {key}")
 
@@ -925,9 +923,8 @@ export(maybesequence,   name='maybesequence',   doc="maybesequence(thing) → 
 export(maybeset,        name='maybeset',        doc="maybeset(thing) → “maybe” boolean predicate, returning True for either None or an instance of “collections.abc.Set”")
 export(maybefieldbase,  name='maybefieldbase',  doc="maybefieldbase(thing) → “maybe” boolean predicate, returning True for either None or an instance of “clu.config.fieldtypes.FieldBase”")
 
-# Assign the modules’ `__all__` using the exporter:
-__all__ = exporter.all_tuple('fields')
-__dir__ = lambda: list(__all__)
+# Assign the modules’ `__all__` and `__dir__` using the exporter:
+__all__, __dir__ = exporter.all_and_dir('fields')
 
 def test():
     from pprint import pprint
