@@ -18,6 +18,7 @@ assigned = pytuple('doc', 'annotations')
 wrap = lambda function: wraps(function, assigned=assigned)
 
 negate = lambda function: wrap(function)(lambda *args, **kwargs: not function(*args, **kwargs))
+reverse = lambda function: wrap(function)(lambda *args, **kwargs: reversed(function(*args, **kwargs)))
 
 negate_doc = """
 ### … You use `negate(function)` thusly:
@@ -194,11 +195,14 @@ dunder_or   = lambda thing, atx: getpyattr(thing, atx, thing)
 
 mro         = lambda thing: getpyattr(typeof(thing), 'mro')
 unwrap      = lambda thing: dunder_or(thing, 'wrapped')
-origin      = lambda thing: dunder_or(typeof(thing), 'origin')
+origin      = lambda thing: typeof(dunder_or(thing, 'origin'))
+rmro        = reverse(mro)
 
 isancestor  = lambda cls, ancestor=object: isclasstype(cls) and (ancestor in mro(cls))
 isorigin    = lambda cls, original=object: isclasstype(cls) and isancestor(origin(cls), typeof(original))
 
+newtype     = lambda name, *bases, **attributes: type(name, tuple(bases) or (object,), dict(attributes))
+
 # ENUM PREDICATES: `isenum(…)` predicate; `enumchoices(…)` to return a tuple
 # of strings naming an enum’s choices (like duh)
 
@@ -219,10 +223,20 @@ def enumchoices(cls):
 
 # PREDICATE LOGCIAL FUNCTIONS: all/any/and/or/xor shortcuts:
 
-predicate_nop   = lambda *things: None
-function_nop    = lambda iterable: None
+@export
+def wrap_value(value):
+    """ Get a “lazified” copy of a value, wrapped in a lamba """
+    wrapper = lambda *args, **kwargs: value
+    wrapper.__wrapped__ = value
+    return wrapper
+
+none_function   = predicate_nop = function_nop = wrap_value(None)
+true_function   = wrap_value(True)
+false_function  = wrap_value(False)
 
 uncallable      = negate(callable) # type: ignore
+hoist           = lambda thing: uncallable(thing) and wrap_value(thing) or thing
+
 pyname          = lambda thing: pyattr(thing, 'name', 'qualname')
 pymodule        = lambda thing: pyattr(thing, 'module', 'package')
 
@@ -339,7 +353,7 @@ def itervariadic(function):
     """ Wrap a variadic function – one with the signature “function(•args)” –
         in logic that allows it to be called with a single-argument iterable
         (“function(iterable)”) with the same effect.
-    
+        
         Screens out strings, bytes-y, and file-path-ish operands without
         iterating or expanding them.
     """
@@ -377,6 +391,18 @@ def listify(*items):
     """ listify(*items) → Return a new list containing all non-`None` arguments """
     return [item for item in items if item is not None]
 
+@export
+@itervariadic
+def union(*items):
+    """ union(*items) → Return the set-union of the contents of all non-`None` arguments """
+    return set().union(item for item in items if item is not None)
+
+# UTILITY ANCESTOR PREDICATES: search through attributes across the MRO of a given type –
+# q.v. “resolve” meta-predicate supra.:
+
+ancestral       = lambda atx, cls, default=tuple(): collator(resolve,            atx, *rmro(cls), default=default)
+ancestral_union = lambda atx, cls, default=tuple(): uniquify(iterchain(ancestral(atx,       cls,  default=default)))
+
 # UTILITY FUNCTIONS: helpers for builtin predicate functions over iterables:
 
 @export
@@ -410,6 +436,7 @@ def slots_for(cls):
 
 # MODULE EXPORTS:
 export(negate,          name='negate',          doc="negate(function) → Negate a boolean function, returning the callable inverse. \n" + negate_doc)
+export(reverse,         name='reverse',         doc="reverse(function) → Reverse an iterating function, returning the reverse of the iterable returned.")
 
 export(ismetaclass,     name='ismetaclass',     doc="ismetaclass(thing) → boolean predicate, True if thing is a metaclass, descending directly from `type`")
 export(isclass,         name='isclass',         doc="isclass(thing) → boolean predicate, True if thing is a class, descending from `object` but not `type`")
@@ -480,14 +507,20 @@ export(finditems,       name='finditems',       doc="finditems(itx, *mappings, d
 
 export(dunder_or,       name='dunder_or',       doc="dunder_or(thing, atx) → Like “getpyattr(…)” only with a default return value of `thing` itself")
 export(mro,             name='mro',             doc="mro(thing) → Return the method resolution order (née “MRO”) tuple for thing (using “type(thing)” for non-classtype operands)")
+export(rmro,            name='rmro',            doc="rmro(thing) → Return the reverse of the method resolution order (née “MRO”) tuple for thing (using “type(thing)” for non-classtype operands)")
 export(unwrap,          name='unwrap',          doc="unwrap(thing) → Return either `thing.__wrapped__` or `thing` for a given `thing`")
 export(origin,          name='origin',          doc="origin(thing) → Return either `typeof(thing).__origin__` or `typeof(thing)` for a given `thing`")
 export(isancestor,      name='isancestor',      doc="isancestor(thing, ancestor=object) → boolean predicate, True if `ancestor` is found in “mro(thing)”")
 export(isorigin,        name='isorigin',        doc="isorigin(thing, original=object) → boolean predicate, True if `original` is an ancestor of “origin(thing)”")
+export(newtype,         name='newtype',         doc="newtype(name, *bases, **attributes) → Shortcut for “type(name, tuple(bases) or (object,), dict(attributes))”")
 
 export(predicate_nop,   name='predicate_nop',   doc="predicate_nop(thing) → boolean predicate that always returns `None`")
 export(function_nop,    name='function_nop',    doc="function_nop(*args) → variadic function always returns `None`")
+export(none_function,   name='none_function',   doc="none_function() → A function that always returns None")
+export(true_function,   name='true_function',   doc="true_function() → A function that always returns True")
+export(false_function,  name='false_function',  doc="false_function() → A function that always returns False")
 export(uncallable,      name='uncallable',      doc="uncallable(thing) → boolean predicate, shortcut for `not callable(thing)`")
+export(hoist,           name='hoist',           doc="hoist(thing) → if “thing” isn’t already callable, turn it into a lambda that returns it as a value (using “wrap_value(…)”).")
 export(pyname,          name='pyname',          doc="pyname(thing) → Return either `__qualname__` or `__name__` from a given `thing`")
 export(pymodule,        name='pymodule',        doc="pymodule(thing) → Return either `__module__` or `__package__` from a given `thing`")
 
@@ -507,6 +540,9 @@ export(predicate_xor,   name='predicate_xor',   doc="predicate_xor(predicate, a,
 export(case_sort,       name='case_sort',       doc="case_sort(string) → Sorting predicate to sort UPPERCASE names first")
 export(isnotnone,       name='isnotnone',       doc="isnotnone(thing) → boolean predicate, return True if “thing” is not None")
 
+export(ancestral,       name='ancestral',       doc="ancestral(atx, cls[, default]) → shortcut for “attr_across(atx, *rmro(cls)[, default])”")
+export(ancestral_union, name='ancestral_union', doc="ancestral_union(atx, cls[, default]) → shortcut for “uniquify(iterchain(attr_across(atx, *mro(cls)[, default])))”")
+
 export(thing_has,       name='thing_has',       doc="thing_has(thing, attribute) → boolean predicate, True if `thing` has “attribute” (in either `__dict__` or `__slots__`)")
 export(class_has,       name='class_has',       doc="class_has(cls, attribute) → boolean predicate, True if `cls` is a class type and has “attribute” (in either `__dict__` or `__slots__`)")
 export(isslotted,       name='isslotted',       doc="isslotted(thing) → boolean predicate, True if `thing` has both an `__mro__` and a `__slots__` attribute")
